@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -141,7 +141,7 @@ public:
     NO_ENTRIES_EXPORTED,                      //  20
     DB_HAS_DUPLICATES,                        //  21
     OK_WITH_ERRORS,                           //  22
-    OK_WITH_VALIDATION_ERRORS                 // 23
+    OK_WITH_VALIDATION_ERRORS                 //  23
   };
 
   PWScore();
@@ -271,6 +271,7 @@ public:
                     int &numValidated, int &numImported, int &numSkipped,
                     int &numPWHErrors, int &numRenamed,
                     int &numNoPolicy,  int &numRenamedPolicies,
+                    int &numShortcutsRemoved,
                     CReport &rpt, Command *&pcommand);
   int ImportKeePassV1TXTFile(const StringX &filename,
                              int &numImported, int &numSkipped, int &numRenamed,
@@ -446,6 +447,10 @@ public:
   size_t GetExpirySize() {return m_ExpireCandidates.size();}
   ExpiredList GetExpired(int idays) {return m_ExpireCandidates.GetExpired(idays);}
 
+  // Yubi support:
+  const unsigned char *GetYubiSK() const;
+  void SetYubiSK(const unsigned char *);
+  
   // Password Policies
   bool IncrementPasswordPolicy(const StringX &sxPolicyName);
   bool DecrementPasswordPolicy(const StringX &sxPolicyName);
@@ -464,6 +469,18 @@ public:
   const std::vector<StringX> &GetEmptyGroups() {return m_vEmptyGroups;}
   bool IsEmptyGroup(const StringX &sxEmptyGroup);
 
+  // Keyboard shortcuts
+  bool AddKBShortcut(const int32 &iKBShortcut, const pws_os::CUUID &uuid);
+  bool DelKBShortcut(const int32 &iKBShortcut, const pws_os::CUUID &uuid);
+  const pws_os::CUUID & GetKBShortcut(const int32 &iKBShortcut);
+  const KBShortcutMap &GetAllKBShortcuts() {return m_KBShortcutMap;}
+  void SetAppHotKey(const int32 &iAppHotKey) {m_iAppHotKey = iAppHotKey;}
+  int32 GetAppHotKey() const {return m_iAppHotKey;}
+
+  uint32 GetHashIters() const;
+  void SetHashIters(uint32 value);
+protected:
+  bool m_isAuxCore; // set in c'tor, if true, never update prefs from DB.  
 private:
   // Database update routines
 
@@ -529,8 +546,9 @@ private:
   unsigned char *m_passkey; // encrypted by session key
   size_t m_passkey_len; // Length of cleartext passkey
 
-  static unsigned char m_session_key[20];
-  static unsigned char m_session_salt[20];
+  uint32 m_hashIters; // for new or currently open db.
+
+  static unsigned char m_session_key[32];
   static unsigned char m_session_initialized;
 
   HANDLE m_lockFileHandle;
@@ -624,6 +642,23 @@ private:
 
   stringT GetXMLPWPolicies();
   PSWDPolicyMap m_MapPSWDPLC;
+
+  KBShortcutMap m_KBShortcutMap;
+  int32 m_iAppHotKey;
+
+  // ValidateKBShortcut() returns true if data modified, false if all OK
+  bool ValidateKBShortcut(int32 &iKBShortcut);
+};
+
+/**
+ * Use this for "other" cores that we want concurrently.
+ * This class does not update the database preferences
+ * when reading a database, so there's no need to save/restore
+ * the "main" db prefs.
+ */
+class PWSAuxCore : public PWScore {
+ public:
+ PWSAuxCore() : PWScore() {m_isAuxCore = true;}
 };
 
 #endif /* __PWSCORE_H */
