@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -10,11 +10,12 @@
 
 #include "stdafx.h"
 #include "passwordsafe.h"
-#include "ThisMfcApp.h"    // For Help
-#include "GeneralMsgBox.h"
+
 #include "DboxMain.h"
+
 #include "AddEdit_PasswordPolicy.h"
 #include "AddEdit_PropertySheet.h"
+#include "GeneralMsgBox.h"
 
 #include "core/core.h"
 #include "core/PwsPlatform.h"
@@ -22,12 +23,8 @@
 #include "core/PWSprefs.h"
 #include "core/PWCharPool.h"
 
-#if defined(POCKET_PC)
-#include "pocketpc/resource.h"
-#else
 #include "resource.h"
 #include "resource3.h"  // String resources
-#endif
 
 using pws_os::CUUID;
 
@@ -178,7 +175,7 @@ BOOL CAddEdit_PasswordPolicy::OnInitDialog()
 
   // Get all password policy names
   std::vector<std::wstring> vNames;
-  M_pDbx()->GetPolicyNames(vNames);
+  GetMainDlg()->GetPolicyNames(vNames);
 
   // Add Default
   CString cs_text(MAKEINTRESOURCE(IDSC_DEFAULT_POLICY));
@@ -217,7 +214,7 @@ BOOL CAddEdit_PasswordPolicy::OnInitDialog()
   if (M_ipolicy() == NAMED_POLICY) {
     if (index != 0) {
       PWPolicy st_pp;
-      M_pDbx()->GetPolicyFromName(StringX((LPCWSTR)M_policyname()), st_pp);
+      GetMainDlg()->GetPolicyFromName(StringX((LPCWSTR)M_policyname()), st_pp);
       M_pwp() = st_pp;
       M_symbols() = st_pp.symbols;
     } else {
@@ -306,9 +303,7 @@ void CAddEdit_PasswordPolicy::OnChanged()
 
 void CAddEdit_PasswordPolicy::OnHelp()
 {
-  CString cs_HelpTopic;
-  cs_HelpTopic = app.GetHelpFileName() + L"::/html/entering_pwd_pp.html";
-  HtmlHelp(DWORD_PTR((LPCWSTR)cs_HelpTopic), HH_DISPLAY_TOPIC);
+  ShowHelp(L"::/html/entering_pwd_pp.html");
 }
 
 bool CAddEdit_PasswordPolicy::ValidatePolicy(CWnd *&pFocus)
@@ -388,13 +383,14 @@ LRESULT CAddEdit_PasswordPolicy::OnQuerySiblings(WPARAM wParam, LPARAM )
   // Have any of my fields been changed?
   switch (wParam) {
     case PP_DATA_CHANGED:
-      if (M_ipolicy()     != M_oldipolicy() ||
+      if (M_ipolicy()     != M_oldipolicy()     ||
           (M_ipolicy()    == SPECIFIC_POLICY &&
-           M_pwp()        != M_oldpwp()) ||
+           M_pwp()        != M_oldpwp())        ||
           (M_ipolicy()    == NAMED_POLICY &&
            M_policyname() != M_oldpolicyname()) ||
           M_iownsymbols() != M_ioldownsymbols() ||
-          M_symbols()     != M_oldsymbols())
+          (M_ipolicy()    != NAMED_POLICY &&
+           M_symbols()    != M_oldsymbols())      )
         return 1L;
       break;
     case PP_UPDATE_VARIABLES:
@@ -745,7 +741,7 @@ void CAddEdit_PasswordPolicy::OnSelectNamedPolicy()
     M_policyname() = (LPCWSTR)cs_text;
 
     PWPolicy st_pp;
-    M_pDbx()->GetPolicyFromName(StringX((LPCWSTR)cs_text), st_pp);
+    GetMainDlg()->GetPolicyFromName(StringX((LPCWSTR)cs_text), st_pp);
     M_pwp() = st_pp;
     M_symbols() = st_pp.symbols;
 
@@ -981,7 +977,7 @@ void CAddEdit_PasswordPolicy::OnNamesComboChanged()
 
   if (index != 0) {
     PWPolicy st_pp;
-    M_pDbx()->GetPolicyFromName(StringX((LPCWSTR)cs_policyname), st_pp);
+    GetMainDlg()->GetPolicyFromName(StringX((LPCWSTR)cs_policyname), st_pp);
     M_pwp() = st_pp;
     M_symbols() = st_pp.symbols;
 
