@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -155,8 +155,8 @@ TIMEINT_ND_SHOWING The length of time the tool tip window remains visible
 // DragBar time interval 
 #define TIMER_DRAGBAR_TIME 100
 
-// Hotkey value ID
-#define PWS_HOTKEY_ID      5767
+// Hotkey value ID to maximum value allowed by Windows for an app.
+#define PWS_HOTKEY_ID      0xBFFF
 
 // Arbitrary string to mean that the saved DB preferences are empty.
 #define EMPTYSAVEDDBPREFS L"#Empty#"
@@ -372,6 +372,13 @@ public:
   bool IsEmptyGroup(const StringX &sxEmptyGroup)
   {return m_core.IsEmptyGroup(sxEmptyGroup);}
   
+  // Entry keyboard shortcuts
+  const KBShortcutMap &GetAllKBShortcuts() const {return m_core.GetAllKBShortcuts();}
+
+  // HashIters relaying
+  uint32 GetHashIters() const {return m_core.GetHashIters();}
+  void SetHashIters(uint32 value) {m_core.SetHashIters(value);}
+
   // Need this to be public
   bool LongPPs(CWnd *pWnd);
 
@@ -466,11 +473,16 @@ public:
   const MapMenuShortcuts &GetMapMenuShortcuts() {return m_MapMenuShortcuts;}
   const std::vector<UINT> &GetExcludedMenuItems() {return m_ExcludedMenuItems;}
   const std::vector<st_MenuShortcut> &GetReservedShortcuts() {return m_ReservedShortcuts;}
+  const unsigned int GetMenuShortcut(const unsigned short int &siVirtKey,
+                                     const unsigned char &cModifier, StringX &sxMenuItemName);
   
   // ClassWizard generated virtual function overrides
   //{{AFX_VIRTUAL(DboxMain)
 
   void ChangeMode(bool promptUser); // r-o <-> r/w
+
+  // If we have processed it returns 0 else 1
+  BOOL ProcessEntryShortcut(WORD &wVirtualKeyCode, WORD &wModifiers);
   
  protected:
   virtual void DoDataExchange(CDataExchange* pDX);  // DDX/DDV support
@@ -704,6 +716,7 @@ public:
   afx_msg void OnOptions();
   afx_msg void OnManagePasswordPolicies();
   afx_msg void OnGeneratePassword();
+  afx_msg void OnYubikey();
   afx_msg void OnSave();
   afx_msg void OnAdd();
   afx_msg void OnAddGroup();
@@ -809,6 +822,7 @@ private:
   bool m_bAlreadyToldUserNoSave;
   bool m_bPasswordColumnShowing;
   bool m_bInRefresh, m_bInRestoreWindows;
+  bool m_bDBInitiallyRO;
   int m_iDateTimeFieldWidth;
   int m_nColumns;
   int m_nColumnIndexByOrder[CItemData::LAST];
@@ -898,7 +912,7 @@ private:
   CInfoDisplay *m_pNotesDisplay;
 
   // Filters
-  bool m_bFilterActive, m_bFilterForStatus, m_bUnsavedDisplayed, m_bExpireDisplayed;
+  bool m_bFilterActive, m_bFilterForStatus, m_bFilterForType, m_bUnsavedDisplayed, m_bExpireDisplayed;
   // Current filter
   st_filters m_currentfilter;
   // Special Show Unsaved Changes filter
