@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -13,15 +13,13 @@
 #include "ThisMfcApp.h"
 #include "DboxMain.h"
 
+#include "HKModifiers.h"
+
 #include "core/PWSprefs.h"
 
-#if defined(POCKET_PC)
-#include "pocketpc/resource.h"
-#else
 #include "resource.h"
 #include "resource2.h"  // Menu, Toolbar & Accelerator resources
 #include "resource3.h"  // String resources
-#endif
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -139,7 +137,6 @@ void DboxMain::SetUpInitialMenuStrings()
   int isubmenu_pos;
   UINT uiCount;
 
-  CHotKeyCtrl cHKC;
   CString sKeyName;
   BOOL brc;
 
@@ -149,7 +146,7 @@ void DboxMain::SetUpInitialMenuStrings()
   // Add user Excluded Menu Items - anything that is a Popup Menu
     ID_FILEMENU, ID_EXPORTMENU, ID_IMPORTMENU, ID_EDITMENU,
     ID_VIEWMENU, ID_FILTERMENU, ID_CHANGEFONTMENU, ID_REPORTSMENU,
-    ID_MANAGEMENU, ID_LANGUAGEMENU, ID_HELPMENU, ID_FINDMENU,
+    ID_MANAGEMENU, ID_LANGUAGEMENU, ID_HELPMENU, ID_FINDMENU, ID_EXPORTENTMENU,
 
   // Plus Exit (2 shortcuts Ctrl+Q and Alt+F4) and Help (F1)
     ID_MENUITEM_EXIT, ID_HELP,
@@ -208,6 +205,7 @@ void DboxMain::SetUpInitialMenuStrings()
       CString csMainMenuItem = tcMenuString;
       int iamp = csMainMenuItem.Find(L'&');
       if (iamp >= 0 && iamp < csMainMenuItem.GetLength() - 1) {
+        st_mst.nControlID = miteminfo.wID;
         st_mst.siVirtKey = csMainMenuItem[iamp + 1];
         m_ReservedShortcuts.push_back(st_mst);
       }
@@ -215,14 +213,17 @@ void DboxMain::SetUpInitialMenuStrings()
   }
 
   // Add 3 special keys F1 (for Help), Ctrl+Q/Alt+F4 (Exit)
+  st_mst.nControlID = ID_MENUITEM_HELP;
   st_mst.siVirtKey = VK_F1;
   st_mst.cModifier = 0;
   m_ReservedShortcuts.push_back(st_mst);
 
+  st_mst.nControlID = ID_MENUITEM_EXIT;
   st_mst.siVirtKey = 'Q';
   st_mst.cModifier = HOTKEYF_CONTROL;
   m_ReservedShortcuts.push_back(st_mst);
 
+  st_mst.nControlID = ID_MENUITEM_EXIT;
   st_mst.siVirtKey = VK_F4;
   st_mst.cModifier = HOTKEYF_ALT;
   m_ReservedShortcuts.push_back(st_mst);
@@ -251,6 +252,9 @@ void DboxMain::SetUpInitialMenuStrings()
   // Do Edit Menu
   InsertShortcuts(pMainMenu, m_MapMenuShortcuts, ID_EDITMENU);
 
+  // Do Export Entry submenu
+  //  InsertShortcuts(pSubMenu, m_MapMenuShortcuts, ID_EXPORTENTMENU);
+
   // Do View Menu
   InsertShortcuts(pMainMenu, m_MapMenuShortcuts, ID_VIEWMENU);
 
@@ -347,20 +351,14 @@ void DboxMain::SetUpInitialMenuStrings()
   // change shortcuts as per preferences
   std::vector<st_prefShortcut> vShortcuts(PWSprefs::GetInstance()->GetPrefShortcuts());
 
-  // We need to convert from PWS shortcut modifiers to MFC modifiers
+  // We need to convert from PWS to Hotkey modifiers
   for (size_t i = 0; i < vShortcuts.size(); i++) {
-    unsigned char cModifier(0), cWindowsMod = vShortcuts[i].cModifier;
-    if ((cWindowsMod & PWS_HOTKEYF_ALT    ) == PWS_HOTKEYF_ALT)
-      cModifier |= HOTKEYF_ALT;
-    if ((cWindowsMod & PWS_HOTKEYF_CONTROL) == PWS_HOTKEYF_CONTROL)
-      cModifier |= HOTKEYF_CONTROL;
-    if ((cWindowsMod & PWS_HOTKEYF_SHIFT  ) == PWS_HOTKEYF_SHIFT)
-      cModifier |= HOTKEYF_SHIFT;
-    if ((cWindowsMod & PWS_HOTKEYF_EXT    ) == PWS_HOTKEYF_EXT)
-      cModifier |= HOTKEYF_EXT;
-    vShortcuts[i].cModifier = cModifier;
+    WORD wPWSModifiers = vShortcuts[i].cModifier;
+    // Translate from CHotKeyCtrl to PWS modifiers
+    WORD wHKModifiers = ConvertModifersPWS2MFC(wPWSModifiers);
+    vShortcuts[i].cModifier = (unsigned char)wHKModifiers;
   }
-
+      
   size_t N = vShortcuts.size();
   for (size_t i = 0; i < N; i++) {
     const st_prefShortcut &stxst = vShortcuts[i];
@@ -503,7 +501,7 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
 
   // This routine changes the text in the menu via "ModifyMenu" and
   // adds the "check" mark via CheckMenuRadioItem for view type and toolbar.
-  // It now tailors the Edit menu depending on whether a Group or Entry is selected.
+  // It then tailors the Edit menu depending on whether a Group or Entry is selected.
   // "EnableMenuItem" is handled by the UPDATE_UI routines
   bool bGroupSelected(false);
   const bool bTreeView = m_ctlItemTree.IsWindowVisible() == TRUE;
@@ -848,10 +846,18 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
         default:
           ASSERT(0);
       }
-      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                             ID_MENUITEM_EXPORTENT2PLAINTEXT, tc_dummy); 
-      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                             ID_MENUITEM_EXPORTENT2XML, tc_dummy);
+
+      CMenu EEsubMenu;
+      CString EEstr;
+      EEsubMenu.CreatePopupMenu();
+      EEstr.LoadString(IDS_EXPORTENT2PLAINTEXT);
+      EEsubMenu.AppendMenu(MF_ENABLED | MF_STRING,
+                           ID_MENUITEM_EXPORTENT2PLAINTEXT, EEstr); 
+      EEstr.LoadString(IDS_EXPORTENT2XML);
+      EEsubMenu.AppendMenu(MF_ENABLED | MF_STRING,
+                          ID_MENUITEM_EXPORTENT2XML, EEstr);
+      EEstr.LoadString(IDS_EXPORTENTMENU);
+      pPopupMenu->AppendMenu(MF_POPUP, (UINT)EEsubMenu.Detach(), EEstr);
 
       if (!bReadOnly && etype_original != CItemData::ET_SHORTCUT)
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
@@ -873,7 +879,6 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
 
 exit:
     SetUpMenuStrings(pPopupMenu);
-    return;
 }
 
 // Helps with MRU by allowing ON_UPDATE_COMMAND_UI
@@ -908,10 +913,11 @@ void DboxMain::OnInitMenuPopup(CMenu* pPopupMenu, UINT, BOOL)
     case ID_EDITMENU:
     case ID_VIEWMENU:
     case ID_FILTERMENU:
+    case ID_EXPORTENTMENU:
     case ID_CHANGEFONTMENU:
     case ID_REPORTSMENU:
     case ID_MANAGEMENU:
-    case ID_HELPMENU:  //main menu items' shortuct should be always updated because keyboard layout could be changed
+    case ID_HELPMENU:  //main menu items' shortcut should be always updated because keyboard layout could be changed
       bDoShortcuts = true;
       break;
     default:
@@ -924,22 +930,17 @@ void DboxMain::OnInitMenuPopup(CMenu* pPopupMenu, UINT, BOOL)
       minfo.dwMenuData == ID_VIEWMENU)
     CustomiseMenu(pPopupMenu, (UINT)minfo.dwMenuData, bDoShortcuts);
 
-  static int iLangPos = -1;
   if (minfo.dwMenuData == ID_MANAGEMENU) {
     // Process the Change Language sub-menu
     // First get its position and then build it
     // If no entries added (no language DLLs or back level language DLLs),
     // disable the menu item.
-    if (iLangPos == -1)
-      iLangPos = app.FindMenuItem(pPopupMenu, ID_LANGUAGEMENU);
-
+    int iLangPos = app.FindMenuItem(pPopupMenu, ID_LANGUAGEMENU);
     if (iLangPos >= 0) {
       CMenu *pSubMenu = pPopupMenu->GetSubMenu(iLangPos);
       const bool brc = ProcessLanguageMenu(pSubMenu);
       pPopupMenu->EnableMenuItem(iLangPos, MF_BYPOSITION | (brc ? MF_ENABLED : MF_GRAYED));
     }
-
-    return;
   }
 
   // http://www4.ncsu.edu:8030/~jgbishop/codetips/dialog/updatecommandui_menu.html
@@ -1059,11 +1060,7 @@ void DboxMain::OnInitMenuPopup(CMenu* pPopupMenu, UINT, BOOL)
 // Called when right-click is invoked in the client area of the window.
 void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
 {
-#if defined(POCKET_PC)
-  const DWORD dwTrackPopupFlags = TPM_LEFTALIGN;
-#else
   const DWORD dwTrackPopupFlags = TPM_LEFTALIGN | TPM_RIGHTBUTTON;
-#endif
 
   BOOL brc;
   CPoint client;
@@ -1428,3 +1425,47 @@ bool DboxMain::ProcessLanguageMenu(CMenu *pPopupMenu)
 
   return true;
 }
+
+const unsigned int DboxMain::GetMenuShortcut(const unsigned short int &siVirtKey,
+                                             const unsigned char &cModifier,
+                                             StringX &sxMenuItemName)
+{
+  unsigned int nControlID(0);
+  sxMenuItemName.empty();
+  MapMenuShortcutsIter inuse_iter;
+
+  st_MenuShortcut st_mst;
+  st_mst.siVirtKey = siVirtKey;
+  st_mst.cModifier = cModifier;
+  
+  inuse_iter = std::find_if(m_MapMenuShortcuts.begin(),
+                            m_MapMenuShortcuts.end(),
+                            already_inuse(st_mst));
+
+  if (inuse_iter != m_MapMenuShortcuts.end()) {
+    nControlID = inuse_iter->first;
+    sxMenuItemName = inuse_iter->second.name.c_str();
+  }
+
+  // std::vector<st_MenuShortcut> m_ReservedShortcuts
+  if (nControlID == 0) {
+    std::vector<st_MenuShortcut>::iterator iter;
+    iter = std::find_if(m_ReservedShortcuts.begin(), m_ReservedShortcuts.end(),
+                        reserved(st_mst));
+
+    if (iter != m_ReservedShortcuts.end()) {
+      nControlID = iter->nControlID;
+      LoadAString(sxMenuItemName, nControlID);
+    }
+  }
+
+  if (!sxMenuItemName.empty()) {
+    // These may have the shortcut hardcoded after a tab character e.g. "\tF1" for Help
+    // Remove the tab and shortcut
+    size_t found = sxMenuItemName.find_first_of(_T("\t"));
+    if (found != StringX::npos)
+      sxMenuItemName.erase(found);
+  }
+
+  return nControlID;
+}
